/***************************************************************************
                               cmudview.cpp
                             -------------------
    begin                : Sat Jun  16 22:10:48 CET 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qsizepolicy.h>
#include <qbrush.h>

// include files for KDE
#include <kapp.h>
#include <kstddirs.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <ktextbrowser.h>

// application specific includes
#include "cmudview.h"
#include "kmuddebug.h"


// FIXME_stephan: filter richtext tags from mud output
// don't let mud users screw the view by doing a "say <b>hahaha</b>

CMudView::CMudView(QWidget *parent, const char *name) : QWidget(parent, name)
{
	_view = new KTextBrowser(this, "CMudView::KTextBrowser",false);
	//_view->setTextFormat(Qt::RichText);
	_view->setTextFormat(Qt::PlainText);
	
	init(QFont("courier"), Qt::red, Qt::black);
	clear();

	
}

CMudView::~CMudView()
{
}

void CMudView::addText(const QString& text)
{
	QString modifiedText;
	int pos;

	modifiedText = text;

	// remove the \rt
	while ( (pos = modifiedText.find("\r")) != -1 )
	{
		// remove the \r only if they are preceded or followed by \n,
		// otherwise replace the \r by a \n for consistent wrapping.
		if ((pos > 1 && modifiedText.mid(pos-1,1) == "\n") ||
		    (pos < ((int)modifiedText.length()-1) && modifiedText.mid(pos+1,1) == "\n"))
		{
			modifiedText.remove(pos, 1);
		}
		else
		{
			modifiedText.replace(pos,1,"\n");
		}
	}

	// replace the \t
	while ( (pos = modifiedText.find("\t")) != -1 )
	{
		QString spaces("        ");
		// standard length of a tab is 8 at position 0, 8, ...
		// otherwise 8 - (position +1) modulo 8 (ie, 4 at pos 3, 1 at pos 6)
		modifiedText.remove(pos, 1);
		modifiedText.insert(pos, spaces.left(8-((pos+1)%8)));
	}

	// remove the \007 (beep)
	while ( (pos = modifiedText.find("\007")) != -1 )
	{
		modifiedText.remove(pos,1);
	}

	/*
	// replace \n with <br>
	while ( (pos = modifiedText.find("\n")) != -1 )
	{
		modifiedText.replace(pos,1,"<br>");
	}

	// replace " " with &nbsp;
	while ( (pos = modifiedText.find(" ")) != -1 )
	{
		modifiedText.replace(pos,1,"&nbsp;");
	}
	*/

	_font = _defaultFont;
	_textColor = _defaultTextColor;
	_bgColor = _defaultBgColor;

	_view->setCurrentFont(_font);
	_view->setColor(_textColor);
	_view->setPaper(QBrush(_bgColor,Qt::SolidPattern));

	/*	modifiedText="<font color=\"" + _textColor.name() + "\" size=" + QString::number(_font.pointSize()) + " face=\"" + _font.family() + "\">" + modifiedText + "</font>"; */
	// append generates new paragraphs which are seperated
	// by a \n. when QT::PlainText is used, text must be
	// inserted using _view->insert()
	/*	_view->append(modifiedText); */
	_view->insert(modifiedText);
	_view->scrollToBottom();

}

void CMudView::init(const QFont font, const QColor textColor, const QColor bgColor)
{
	kdDebug(DAREA_UI_VIEW) << "init: font(" << font.family() << ", " << font.pointSize()
	                       <<"pt), color(" << textColor.name() << "/" << bgColor.name() << ")" << endl;
	_defaultFont = font;
	_defaultTextColor = textColor;
	_defaultBgColor = bgColor;
	_font = _defaultFont;
	_textColor = _defaultTextColor;
	_bgColor = _defaultBgColor;
}

void CMudView::clear()
{
	_font = _defaultFont;
	_textColor = _defaultTextColor;
	_bgColor = _defaultBgColor;
	_view->clear();
}

void CMudView::setFont(QFont font, bool useStyle)
{
	if (!useStyle)
	{
		font.setUnderline(_defaultFont.underline());
		font.setItalic(_defaultFont.italic());
		font.setBold(_defaultFont.bold());
	}
	
	_defaultFont = font;

	kdDebug(DAREA_UI_VIEW) << "setFont: font(" << _defaultFont.family() << ", " << _defaultFont.pointSize() <<"pt)" << endl;

}

QFont CMudView::font()
{
	return _defaultFont;
}

void CMudView::setUnderline(bool underline)
{
	_defaultFont.setUnderline(underline);
}
	
bool CMudView::underline()
{
	return _defaultFont.underline();
}

void CMudView::setItalic(bool italic)
{
	_defaultFont.setItalic(italic);
}

bool CMudView::italic()
{
	return _defaultFont.italic();
}

void CMudView::setBold(bool bold)
{
	_defaultFont.setBold(bold);
}

bool CMudView::bold()
{
	return _defaultFont.bold();
}

void CMudView::setTextColor(QColor color)
{
	_defaultTextColor = color;
}

QColor CMudView::textColor()
{
	return _defaultTextColor;
}

void CMudView::setBgColor(QColor color)
{
	_defaultBgColor = color;
}

QColor CMudView::bgColor()
{
	return _defaultBgColor;
}

void CMudView::resizeEvent(QResizeEvent* e)
{
	_view->resize(e->size().width(), e->size().height());
}

